'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.WEEK = exports.DAY = exports.MINUTE60 = undefined;

var _lodash = require('lodash.range');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.padstart');

var _lodash4 = _interopRequireDefault(_lodash3);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _objectAccess = require('object-access');

var _objectAccess2 = _interopRequireDefault(_objectAccess);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const date = d => new Date(d);
const padNumber = number => (0, _lodash4.default)('' + number, 2, '0');

const dayString = time => (0, _moment2.default)(time).format('YYYY-MM-DD');
const minuteString = time => (0, _moment2.default)(time).format('YYYYMMDDHHmm') + '00';
const parseMinute = timestring => (0, _moment2.default)(timestring, 'YYYYMMDDHHmm').toDate();
const parseDay = timestring => (0, _moment2.default)(timestring, 'YYYY-MM-DD').toDate();

// MINUTE5: {
//   key: 'minute5',
//   url: code => `http://ifzq.gtimg.cn/appstock/app/kline/mkline?param=${code},m5,,10000`,
//   prop: 'm5',
//   formatTime: minuteString,
//   parseTime: parseMinute
// },
//
// MINUTE15: {
//   key: 'minute15',
//   url: code =>  `http://ifzq.gtimg.cn/appstock/app/kline/mkline?param=${code},m15,,10000`,
//   prop: 'm15',
//   formatTime: minuteString,
//   parseTime: parseMinute
// },
//
// MINUTE30: {
//   key: 'minute30',
//   url: code =>  `http://ifzq.gtimg.cn/appstock/app/kline/mkline?param=${code},m30,,10000`,
//   prop: 'm30',
//   formatTime: minuteString,
//   parseTime: parseMinute
// },

// Period: 4 months
const MINUTE60 = exports.MINUTE60 = {
  // Trading hours in 4 months: 4 * (31 / 7 * 5) * 4 ~~= (<) 360
  url(code, [, before = ''], limit = 360) {
    return `http://ifzq.gtimg.cn/appstock/app/kline/mkline?param=${code},m60,${before},${limit}`;
  },
  prop: 'm60',
  formatTime: minuteString,
  parseTime: parseMinute,
  closest(to) {
    const nextQDate = (0, _moment2.default)(to).add(4, 'M');
    const monthsToSub = nextQDate.month() % 4;
    return nextQDate.subtract(monthsToSub, 'M').date(0).hours(0).minutes(0).seconds(0).milliseconds(0);
  },
  stepBack(moment) {
    return moment.subtract(4, 'M');
  },
  // Into
  map([from, to]) {
    to = to || new Date();
    const before = MINUTE60.closest(to);
    const ranges = [];

    do {
      ranges.unshift([, MINUTE60.formatTime(before)]);
      MINUTE60.stepBack(before);
    } while (before > from);

    return ranges;
  }
};

// Period: 1 year
const DAY = exports.DAY = {
  // [from, to] is a left-close and right-OPEN region
  // limit: use 321 to prevent qq cache
  url: (code, [from = '', before = ''], limit = 321) => `http://web.ifzq.gtimg.cn/appstock/app/fqkline/get?param=${code},day,${from},${before},${limit},qfq`,
  // The key of the response data
  prop: (data, { code }) => (0, _objectAccess2.default)(data, ['data', code, 'day']) || (0, _objectAccess2.default)(data, ['data', code, 'qfqday']) || [],

  // Transform request time -> response time format
  // Date -> 2017-09-01
  formatTime: dayString,
  parseTime: parseDay,
  // Splits time ranges into each year
  map([from, to]) {
    to = to || new Date();
    const fromYear = from.getFullYear();
    let toYear = fromYear;

    while (new Date(String(toYear + 1)) <= to) {
      toYear++;
    }

    return (0, _lodash2.default)(fromYear, toYear + 1).map(year => [`${year}-01-01`, `${year + 1}-01-01`]);
  }
};

// Period: 6 years, 6 * 54 = 324 < 340
const WEEK = exports.WEEK = {
  key: 'week',
  url: (code, [from = '', before = ''], limit = 340) => `http://web.ifzq.gtimg.cn/appstock/app/fqkline/get?param=${code},week,${from},${before},${limit},qfq`,
  prop: 'qfqweek',
  formatTime: dayString,
  parseTime: parseDay,
  closest(to) {
    // Ends with the first friday of the next year
    return (0, _moment2.default)(to).add(1, 'y').month(0).day(5).hours(0).minutes(0).seconds(0).milliseconds(0);
  },
  stepBack(moment) {
    return moment.subtract(6 * 54, 'w');
  },
  map([from, to]) {
    to = to || new Date();
    const before = WEEK.closest(to);
    const ranges = [];
    let formated;

    do {
      formated = WEEK.formatTime(before);
      WEEK.stepBack(before);
      ranges.unshift([WEEK.formatTime(before), formated]);
    } while (before > from);

    return ranges;
  }
};

//
// MONTH: {
//   key: 'month',
//   url: code =>
//   `http://web.ifzq.gtimg.cn/appstock/app/fqkline/get?_var=kline_monthqfq&param=${code},month,,,320,qfq`,
//   prop: 'qfqmonth',
//   formatTime: dayString,
//   parseTime: parseDay,
//   replace: string => string.replace(/^kline_monthqfq=/, '')
// }